# -*- coding: utf-8 -*-
#                    _
#     /\            | |
#    /  \   _ __ ___| |__   ___ _ __ _   _
#   / /\ \ | '__/ __| '_ \ / _ \ '__| | | |
#  / ____ \| | | (__| | | |  __/ |  | |_| |
# /_/    \_\_|  \___|_| |_|\___|_|   \__, |
#                                     __/ |
#                                    |___/
# Copyright (C) 2017 Anand Tiwari
#
# Email:   anandtiwarics@gmail.com
# Twitter: @anandtiwarics
#
# This file is part of ArcherySec Project.

from __future__ import unicode_literals

from django.db import models
from django.utils import timezone

from user_management.models import Organization, UserProfile


class PentestScanDb(models.Model):
    scan_url = models.URLField(blank=True)
    scan_id = models.TextField(blank=True)
    total_vul = models.IntegerField(blank=True, null=True)
    critical_vul = models.IntegerField(blank=True, null=True)
    high_vul = models.IntegerField(blank=True, null=True)
    medium_vul = models.IntegerField(blank=True, null=True)
    low_vul = models.IntegerField(blank=True, null=True)
    project = models.ForeignKey(
        "projects.ProjectDb", on_delete=models.CASCADE, null=True
    )
    date_time = models.DateTimeField(auto_now=True, blank=True, null=True)
    pentest_type = models.TextField(blank=True, null=True)
    created_time = models.DateTimeField(
        auto_now=True,
        blank=True,
    )
    created_by = models.ForeignKey(
        UserProfile,
        on_delete=models.SET_NULL,
        null=True,
        related_name="pentest_scan_db_created",
    )
    updated_by = models.ForeignKey(
        UserProfile,
        related_name="pentest_scan_db_updated",
        on_delete=models.SET_NULL,
        null=True,
    )
    is_active = models.BooleanField(default=True)
    organization = models.ForeignKey(Organization, on_delete=models.CASCADE, default=1)


class PentestScanResultsDb(models.Model):
    vuln_id = models.TextField(blank=True)
    scan_id = models.TextField(blank=True)
    date_time = models.DateTimeField(auto_now=True, blank=True, null=True)
    rescan_id = models.TextField(blank=True, null=True)
    vuln_name = models.TextField(blank=True, null=True)
    severity = models.TextField(blank=True, null=True)
    severity_color = models.TextField(blank=True, null=True)
    vuln_url = models.TextField(blank=True, null=True)
    scan_url = models.TextField(blank=True)
    description = models.TextField(blank=True, null=True)
    solution = models.TextField(blank=True, null=True)
    request_header = models.TextField(blank=True, null=True)
    response_header = models.TextField(blank=True, null=True)
    reference = models.TextField(blank=True, null=True)
    vuln_status = models.TextField(null=True, blank=True)
    note = models.TextField(null=True, blank=True)
    project = models.ForeignKey(
        "projects.ProjectDb", on_delete=models.CASCADE, null=True
    )
    Poc_Img = models.ImageField(null=True, blank=True)
    poc_description = models.TextField(null=True, blank=True)
    pentest_type = models.TextField(blank=True, null=True)
    created_time = models.DateTimeField(
        auto_now=True,
        blank=True,
    )
    created_by = models.ForeignKey(
        UserProfile,
        on_delete=models.SET_NULL,
        null=True,
        related_name="pentest_scan_result_db_created",
    )
    updated_by = models.ForeignKey(
        UserProfile,
        related_name="pentest_scan_result_db_updated",
        on_delete=models.SET_NULL,
        null=True,
    )
    is_active = models.BooleanField(default=True)
    organization = models.ForeignKey(Organization, on_delete=models.CASCADE, default=1)


class VulnerabilityData(models.Model):
    vuln_data_id = models.TextField(blank=True)
    vuln_name = models.TextField(blank=True)
    vuln_description = models.TextField(blank=True)
    vuln_severity = models.TextField(blank=True)
    vuln_remediation = models.TextField(blank=True)
    vuln_references = models.TextField(blank=True)
    date_time = models.DateTimeField(auto_now=True, blank=True, null=True)
    created_time = models.DateTimeField(
        auto_now=True,
        blank=True,
    )
    created_by = models.ForeignKey(
        UserProfile,
        on_delete=models.SET_NULL,
        null=True,
        related_name="vulnerability_data_db_created",
    )
    updated_by = models.ForeignKey(
        UserProfile,
        related_name="vulnerability_data_db_updated",
        on_delete=models.SET_NULL,
        null=True,
    )
    is_active = models.BooleanField(default=True)
    organization = models.ForeignKey(Organization, on_delete=models.CASCADE, default=1)
